//simple library to generate graphs. by mzuenni
#include <deque>
#include <vector>
#include <map>
#include <optional>
//#include <ranges>
#include <random>
#include <cassert>
#include <algorithm>
#include <iostream>

namespace icpc_gen_graph {

	struct NoData {};

	template<typename N = NoData>
	struct NodeWrapper {
		const int id;
		N& data;
		N& operator*() {return data;}
		const N& operator*() const {return data;}
	};

	template<typename E = NoData>
	struct EdgeWrapper {
		const int from, to;
		E& data;
		E& operator*() {return data;}
		const E& operator*() const {return data;}
	};

	template<typename E, typename N, bool DIR>
	class GraphType {
		template<typename, typename, bool> friend class GraphType;

		std::deque<N> nodes;
		std::deque<E> edges;
		std::vector<size_t> freeEdges;
		std::deque<std::map<int, size_t>> adj;
		int offset;

		std::optional<size_t> edgeId(int from, int to) const {
			if (from < offset || from >= offset + nodeCount()) return {};
			auto it = adj[from - offset].find(to);
			if (it == adj[from - offset].end()) return {};
			return it->second;
		}

		GraphType(const std::deque<N>& _nodes, int _offset) :
			nodes(_nodes), edges(0), adj(_nodes.size()), offset(_offset) {}

	public:
		GraphType(int from, int to) :
			nodes(to-from), edges(0), adj(to-from), offset(from) {}
		GraphType(int n) :
			GraphType(0, n) {}

		int nodeCount() const {
			return static_cast<int>(nodes.size());
		}

		int edgeCount() const {
			return static_cast<int>(edges.size())
				 - static_cast<int>(freeEdges.size());
		}

		int getOffset() const {
			return offset;
		}

		std::optional<NodeWrapper<const N>> getNode(int id) const {
			if (id < offset || id >= offset + nodeCount()) return {};
			return NodeWrapper(id, nodes[id - offset]);
		}

		int getDegree(int id) {
			if (id < offset || id >= offset + nodeCount()) return 0;
			return static_cast<int>(adj[id - offset].size());
		}

		std::optional<NodeWrapper<N>> getNode(int id) {
			if (id < offset || id >= offset + nodeCount()) return {};
			return NodeWrapper(id, nodes[id - offset]);
		}

		std::optional<EdgeWrapper<const E>> getEdge(int from, int to) const {
			auto id = edgeId(from, to);
			if (!id) return {};
			size_t x = *id;
			return EdgeWrapper(from, to, edges[*id]);
		}

		std::optional<EdgeWrapper<E>> getEdge(int from, int to) {
			auto id = edgeId(from, to);
			if (!id) return {};
			return EdgeWrapper(from, to, edges[*id]);
		}

		bool hasEdge(int from, int to) const {
			return edgeId(from, to).has_value();
		}

		EdgeWrapper<E> addEdge(int from, int to, const E& data = {}) {
			auto id = edgeId(from, to);
			if (!id) {
				if (!freeEdges.empty()) {
					id = freeEdges.back();
					freeEdges.pop_back();
					edges[*id] = data;
				} else {
					id = edgeCount();
					edges.push_back(data);
				}
				adj[from-offset][to] = *id;
				if constexpr (!DIR) if (from != to) adj[to-offset][from] = *id;
			} else {
				edges[*id] = data;
			}
			return EdgeWrapper(from, to, edges[*id]);
		}

		bool removeEdge(int from, int to) {
			if (from >= offset && from < offset + nodeCount()) {
				auto it = adj[from-offset].find(to);
				if (it != adj[from-offset].end()) {
					size_t id = it->second;
					adj[from-offset].erase(it);
					if constexpr (!DIR) {
						it = adj[to-offset].find(from);
						assert(it != adj[to-offset].end());
						assert(id == it->second);
						adj[to-offset].erase(it);
					}
					freeEdges.push_back(id);
					return true;
				}
			}
			return false;
		}

		bool removeEdge(const EdgeWrapper<E>& e) {
			return removeEdge(e.from, e.to);
		}

		std::vector<EdgeWrapper<const E>> getEdges(int id) const {
			std::vector<EdgeWrapper<const E>> res;
			for (auto [to, edge] :  adj[id-offset]) {
				res.emplace_back(id, to, edges[edge]);
			}
			return res;
		}

		std::vector<EdgeWrapper<E>> getEdges(int id) {
			std::vector<EdgeWrapper<E>> res;
			for (auto [to, edge] :  adj[id-offset]) {
				res.emplace_back(id, to, edges[edge]);
			}
			return res;
		}

		std::vector<EdgeWrapper<const E>> getEdges() const {
			std::vector<EdgeWrapper<const E>> res;
			for (int i = 0; i < nodeCount(); i++) {
				auto tmp = getEdges(i+offset);
				for (auto e : tmp) {
					if (DIR || e.from <= e.to) {
						res.push_back(e);
					}
				}
			}
			return res;
		}

		std::vector<EdgeWrapper<E>> getEdges() {
			std::vector<EdgeWrapper<E>> res;
			for (int i = 0; i < nodeCount(); i++) {
				auto tmp = getEdges(i+offset);
				for (auto e : tmp) {
					if (DIR || e.from <= e.to) {
						res.push_back(e);
					}
				}
			}
			return res;
		}

		GraphType<E, N, true> reverse() const {
			static_assert(DIR, "reverse() is only available on directed graphs!");
			GraphType<E, N, true> res(nodes, offset);
			for (const auto& e : getEdges()) {
				res.addEdge(e.to, e.from, *e);
			}
			return res;
		}

		GraphType<E, N, false> undirected() const {
			static_assert(DIR, "undirected() is only available on directed graphs!");
			GraphType<E, N, false> res(nodes, offset);
			for (const auto& e : getEdges()) {
				res.addEdge(e.from, e.to, *e);
			}
			return res;
		}

		template<class PRED>
		GraphType<E, N, true> directed(const PRED&& p) const {
			static_assert(!DIR, "directed() is only available on undirected graphs!");
			GraphType<E, N, true> res(nodes, offset);
			for (const auto& e : getEdges()) {
				if (p(e.from, e.to)) res.addEdge(e.from, e.to, *e);
				if (p(e.to, e.from)) res.addEdge(e.to, e.from, *e);
			}
			return res;
		}

		GraphType<E, N, true> directed() const {
			return directed([](int a, int b){
				(void)a;
				(void)b;
				return true;
			});
		}

		template<class URBG>
		GraphType<E, N, DIR> permutate(URBG&& rng) const {
			std::vector<int> perm(nodeCount());
			std::iota(perm.begin(), perm.end(), offset);
			std::shuffle(perm.begin(), perm.end(), rng);
			GraphType<E, N, DIR> res(offset, offset + nodeCount());
			for (int i = 0; i < nodeCount(); i++) {
				res.nodes[perm[i] - offset] = nodes[i];
			}
			for (const auto& e : getEdges()) {
				res.addEdge(perm[e.from - offset], perm[e.to - offset], *e);
			}
			return res;
		}

		GraphType<E, N, DIR> range(int from, int to) const {
			assert(from <= to);
			GraphType<E, N, DIR> res(from, to);
			from = std::max(0, from - offset);
			to = std::min(nodeCount(), to - offset);
			for (int i = from; i < to; i++) {
				res.nodes[i] = nodes[i];
				for (const auto& e : adj[i]) {
					res.addEdge(i + offset, e.first, edges[e.second]);
				}
			}
			return res;
		}

		GraphType<E, N, DIR>& setOffset(int _offset) {
			for (auto& n : adj) {
				std::map<int, size_t> old;
				swap(n, old);
				for (auto& e : old) {
					n.emplace(e.first - offset + _offset, e.second);
				}
			}
			offset = _offset;
			return *this;
		}

		GraphType<E, N, DIR>& normalize() {
			return setOffset(0);
		}

		GraphType<E, N, DIR> copy() const {
			return *this;
		}

		GraphType<E, N, DIR> product(const GraphType<E, N, DIR>& o) const {
			GraphType<E, N, DIR> res(nodeCount() * o.nodeCount());
			auto newId = [&](int a, int b){
				return (b-o.offset) * nodeCount() + (a-offset);
			};
			auto a = getEdges();
			auto b = o.getEdges();
			for (const auto& e1 : a) {
				for (const auto& e2 : b) {
					int from = newId(e1.from, e2.from);
					int to = newId(e1.to, e2.to);
					res.addEdge(from, to);
				}
			}
			return res;
		}

		GraphType<E, N, DIR> cartesian(const GraphType<E, N, DIR>& o) const {
			GraphType<E, N, DIR> res(nodeCount() * o.nodeCount());
			auto newId = [&](int a, int b){
				return (b-o.offset) * nodeCount() + (a-offset);
			};
			for (const auto& e : o.getEdges()) {
				for (int i = 0; i < nodeCount(); i++) {
					int from = newId(i + offset, e.from);
					int to = newId(i + offset, e.to);
					res.addEdge(from, to);
				}
			}
			for (const auto& e : getEdges()) {
				for (int i = 0; i < o.nodeCount(); i++) {
					int from = newId(e.from, i + o.offset);
					int to = newId(e.to, i + o.offset);
					res.addEdge(from, to);
				}
			}
			return res;
		}

		GraphType<E, N, DIR> strongProduct(const GraphType<E, N, DIR>& o) const {
			return o.cartesian(*this).combine(o.product(*this));
		}

		GraphType<E, N, DIR> lineGraph() const {
			static_assert(!DIR, "lineGraph() is only available on undirected graphs!");
			GraphType<int, NoData, DIR> tmp(offset, offset + nodeCount());
			int nextEdge = 0;
			for (const auto e : getEdges()) {
				tmp.addEdge(e.from, e.to, nextEdge);
				nextEdge++;
			}
			GraphType<E, N, DIR> res(nextEdge);
			for (int i = 0; i < tmp.nodeCount(); i++) {
				auto edges = tmp.getEdges(i);
				for (const auto& e1 : edges) {
					for (const auto& e2 : edges) {
						if (*e1 < *e2) res.addEdge(*e1, *e2);
					}
				}
			}
			return res;
		}

		GraphType<E, N, DIR>& combine(const GraphType<E, N, DIR>& o) {
			assert(&o != this);
			while (offset > o.offset) {
				offset--;
				adj.emplace_front();
				nodes.push_front(o.getNode(offset).value().data);
			}
			while (offset + nodeCount() < o.offset + nodeCount()) {
				adj.emplace_back();
				nodes.push_back(o.getNode(offset + nodeCount()).value().data);
			}
			for (const auto& e : o.getEdges()) {
				addEdge(e.from, e.to, *e);
			}
			return *this;
		}

		GraphType<E, N, DIR>& append(const GraphType<E, N, DIR>& o) {
			assert(&o != this);
			int diff = nodeCount() + offset - o.offset;
			adj.resize(nodeCount() + o.nodeCount());
			for(int i = 0; i < o.nodeCount(); ++i) {
			    nodes.push_back(N());
			}
			for (const auto& e : o.getEdges()) {
				addEdge(e.from + diff, e.to + diff, *e);
			}
			return *this;
		}

	};

	template<typename E = NoData, typename N = NoData>
	using Graph = GraphType<E, N, false>;

	template<typename E = NoData, typename N = NoData>
	using DiGraph = GraphType<E, N, true>;

	// n independed vertices in range [0, n)
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> independent(int n) {
		return Graph<E, N>(n);
	}

	// n connected vertices in range [0, n)
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> clique(int n) {
		// could be implemented as linegraph of a star.
		Graph<E> res(n);
		for (int i = 0; i < n; i++) {
			for (int  j = 0; j < i; j++) {
				res.addEdge(i, j);
			}
		}
		return res;
	}

	// complete bipartite graph with partition [0, n) and [n, n+m)
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> bipartite(int n, int m) {
		Graph<E, N> res(n+m);
		for (int i = 0; i < n; i++) {
			for (int j = 0; j < m; j++) {
				res.addEdge(i, n + j);
			}
		}
		return res;
	}

	// star with root = 0 and chilrden [1, leafes]
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> star(int leafes) {
		return bipartite<E, N>(1, leafes);
	}

	// path 0-1-2-...-edges with vertices [0, edges]
	template<typename E = NoData, typename N = NoData, bool DIR = false>
	GraphType<E, N, DIR> path(int edges) {
		GraphType<E, N, DIR> res(edges+1);
		for (int i = 0; i < edges; i++) {
			res.addEdge(i, i + 1);
		}
		return res;
	}

	// path 0-1-2-...-n-1-0 with vertices [0, n)
	template<typename E = NoData, typename N = NoData, bool DIR = false>
	GraphType<E, N, DIR> cycle(int n) {
		auto p = path(n-1);
		p.addEdge(n - 1, 0);
		return p;
	}

	// wheel with root = 1 and outer vertices 1-2-...-outer-1 with vertices [0, outer]
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> wheel(int outer) {
		return star(outer).combine(path(outer).setOffset(1));
	}

	// lolly graph with clique [0, c) and path with p edges [c, c+p)
	// c is the vertex where clique and path are connected
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> lolly(int c, int p) {
		return clique(c).combine(path(p).setOffset(c-1));
	}

	// n times m grid
	// 0-1-...-n-1
	// | |      |
	// n- ... -2*n-1
	// |        |
	// .        .
	//          |
	//        -n*m-1
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> grid(int n, int m) {
		return path(n-1).cartesian(path(m-1));
	}
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> grid(int n) {
		return grid(n, n);
	}

	// n dimensional hyper cube with vertices [0, 2^dimension)
	// a-b <=> |a^b|=1
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> hypercube(int dimension) {
		if (dimension < 2) return clique(dimension+1);
		return hypercube(dimension-1).cartesian(clique(2));
	}

	// complete tree with root at 0 and all subtrees are consecutive
	// vertices in [0, (children^{level+1}-1)/(children-1))
	template<typename E = NoData, typename N = NoData, bool DIR = false>
	GraphType<E, N, DIR> completeTree(int level, int children = 2) {
		if (level <= 2) return star(children);
		GraphType<E, N, DIR> res(1);
		GraphType<E, N, DIR> sub = completeTree(level - 1, children);
		for (int i = 0; i < children; i++) {
			int next = res.nodeCount();
			res.append(sub);
			res.addEdge(0, next);
		}
		return res;
	}

	// complete boolean lattice i->j <=> i|j==j
	// vertices in [0, 2^level)
	template<typename E = NoData, typename N = NoData, bool DIR = false>
	GraphType<E, N, DIR> booleanLattice(int level) {
		GraphType<E, N, DIR> res(1 << level);
		for (int i = 0; i < res.nodeCount(); i++) {
			for (int j = 1; j < res.nodeCount(); j*=2) {
				if ((i | j) == i) {
					res.addEdge(i|j, i);
				}
			}
		}
		return res;
	}

	// prefix of the rado graph i->j <=> i<j && j[i] == true;
	// vertices in [0, n)
	template<typename E = NoData, typename N = NoData, bool DIR = false>
	GraphType<E, N, DIR> rado(int n) {
		GraphType<E, N, DIR> res(n);
		for (int i = 0; i < n; i++) {
			for (int j = 0; (1 << j) < i; j++) {
				if ((i | (1 << j)) == i) {
					res.addEdge(i, j);
				}
			}
		}
		return res;
	}

	// generate a tree from a pruefer code
	// values should be in [0, sz(code)+2)
	template<typename E = NoData, typename N = NoData>
	Graph<E, N> pruefer(const std::vector<int>& code) {
		Graph<E, N> res(code.size() + 2);
		std::vector<int> count(res.nodeCount());
		for (int x : code) count[x]++;
		int current = -1;
		int skip = -1;
		auto getNext = [&](){
			int oldSkip = skip;
			skip = -1;
			if (oldSkip >= 0 && oldSkip <= current) return oldSkip;
			for (current++; count[current] > 0; current++) {}
			return current;
		};
		for (int x : code) {
			int y = getNext();
			res.addEdge(x, y);
			count[x]--;
			if (count[x] == 0) skip = x;
		}
		int x = getNext();
		int y = getNext();
		res.addEdge(x, y);
		return res;
	}

	// generate a random tree with vertices [0, n)
	template<typename E = NoData, typename N = NoData, class URBG>
	Graph<E, N> randomTree(int n, URBG&& rng) {
		std::vector<int> code(n - 2);
		std::uniform_int_distribution<int> distr(0, n-1);
		for (int& x : code) x = distr(rng);
		return pruefer(code);
	}

	// generate a random graph (Erdös-Renyi)
	// each edge exists with probability p
	template<typename E = NoData, typename N = NoData, bool DIR = false, class URBG>
	GraphType<E, N, DIR> randomGraph(int n, double p, URBG&& rng) {
		assert(p >= 0 && p <= 1);
		GraphType<E, N, DIR> res(n);
		std::uniform_real_distribution<double> distr(0, 1);
		for (int i = 0; i < n; i++) {
			for (int j = 0; j < i; j++) {
				if (distr(rng) <= p) res.addEdge(i, j);
				if constexpr(DIR) {
					if (distr(rng) <= p) res.addEdge(j, i);
				}
			}
		}
		return res;
	}

	// generate a random growing graph
	// when adding vertex i the edge i-j has probability scale*deg(j)/m
	template<typename E = NoData, typename N = NoData, class URBG>
	Graph<E, N> randomGrowingGraph(int n, double scale, URBG&& rng) {
		Graph<E, N> res(n);
		int m = 0;
		for (int i = 0; i < n; i++) {
			for (int j = 0; j < i; j++) {
				if (std::uniform_real_distribution<double>(0, m)(rng) <= res.getDegree(j) * scale) {
					res.addEdge(i, j);
					m++;
				}
			}
		}
		return res;
	}

}
