#include <compare>
#include <iomanip>
#include <iostream>
#include <numeric>
#include <vector>
using namespace std;
using integer = __int128;

integer isqrt(integer n) {
	integer k = sqrt((long double) n);
	while (k*k <= n) k++;
	while (k*k > n) k--;
	return k;
}

inline ostream &operator<<(ostream &out, integer x) {
	if (x < 0) out << "-", x = -x;
	long long B = 1e18;
	if (x < B) return out << (long long) x;
	else return out << (long long) (x/B) << setw(18) << setfill('0') << (long long) (x%B);
}

struct fraction {
	integer a, b;

	fraction(integer c, integer d = 1): a(c), b(d) {
		integer g = gcd(a,b);
		a /= g, b /= g;
		if (b < 0) a = -a, b = -b;		
	}

	fraction operator+(const fraction &f) const {
		return fraction(a*f.b + f.a*b, b*f.b);
	}

	fraction operator-(const fraction &f) const {
		return fraction(a*f.b - f.a*b, b*f.b);
	}

	fraction operator-() const {
		return fraction(-a, b);
	}

	fraction operator*(const fraction &f) const {
		return fraction(a*f.a, b*f.b);
	}

	fraction operator/(const fraction &f) const {
		return fraction(a*f.b, b*f.a);
	}

	auto operator<=>(const fraction &f) const {
		return a*f.b <=> f.a*b;
	}

	bool operator==(const fraction &f) const = default;

	friend ostream &operator<<(ostream &out, const fraction &f) {
		if (f.b == 1) return out << f.a;
		return out << f.a << "/" << f.b;
	}
};

// Numbers of the form u + v*sqrt(D), where D is a non-negative integer and u
// and v are fractions. Every such number has a canonical representation where
// D is either 0 (in which case the number is rational) or D is square free. We
// find this unique representation by trial division in O(cbrt(D)). This is
// fast enough, as all coordinates are in [-1000,1000], so that D is at most on
// the order of 10^12 (see the circle intersection code below).
struct quadratic_irrational {
	fraction u, v;
	integer D;

	quadratic_irrational(fraction a, fraction b, integer C): u(a), v(b), D(C) {
		if (D == 0) v = 0;
		// Factor out squares of primes by trial division until cbrt(D).
		for (integer p = 2; p*p*p <= D; p++) {
			while (D % (p*p) == 0) v = v*p, D /= p*p;
		}
		// Now D has at most two prime factors, so it is enough to check if it
		// is a square number.
		integer q = isqrt(D);
		if (q*q == D) v = v*q, D = 1;
		if (D == 1) u = u+v, v = 0, D = 0;
	}

	bool operator==(const quadratic_irrational &q) const = default;

	friend ostream &operator<<(ostream &out, const quadratic_irrational &x) {
		if (x.v == 0) return out << x.u;
		if (x.u == 0) out << x.v;
		else out << x.u << " " << (x.v > 0 ? "+"  : "-") << " " << (x.v > 0 ? x.v : -x.v);
		return out << "*sqrt(" << x.D << ")";
	}
};

struct circle {
	integer x, y, r;
	auto operator<=>(const circle &c) const = default;
};

struct intersection {
	quadratic_irrational x, y;
	bool operator==(const intersection &i) const = default;
};

// Intersect two circles and add their intersection points to the given vector.
// The coordinates and circle radii are bounded by B = 1000 in absolute value,
// so that both D and the numerators/denominators of the fractions u and v in
// the intersection points are on the order of B^4 = 10^12. Note that the
// normalization during the construction of the quadratic irrationals does not
// change this, as v's numerator is initially only on the order of B^2, so it
// can at most grow to B^4 as well.
void intersect(circle a, circle b, vector<intersection> &v) {
	integer dx = b.x-a.x, dy = b.y-a.y;
	integer d = dx*dx + dy*dy;
	if (d > (a.r+b.r)*(a.r+b.r)) return; // disjoint
	if (d < (a.r-b.r)*(a.r-b.r)) return; // inside

	integer C = a.r*a.r - b.r*b.r + d;
	integer D = 4*d*a.r*a.r - C*C;
	
	quadratic_irrational x1(fraction(a.x) + fraction(C*dx,2*d), fraction(-dy,2*d), D);
	quadratic_irrational y1(fraction(a.y) + fraction(C*dy,2*d), fraction( dx,2*d), D);
	
	quadratic_irrational x2(fraction(a.x) + fraction(C*dx,2*d), fraction( dy,2*d), D);
	quadratic_irrational y2(fraction(a.y) + fraction(C*dy,2*d), fraction(-dx,2*d), D);
	
	v.emplace_back(x1, y1), v.emplace_back(x2, y2);
}
